﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Reflection;
using System.IO;

using PluginHandler.Interfaces;

namespace PluginHandler
{
    public static class PluginHostProvider
    {
        private const string PluginDllLocation = "Groups";
        private const string Version = "1.0.0.0";

        private static List<PluginHost> pluginHosts = null;

        public static IPlugin GetPlugin(String pSuffix)
        {
            List<Assembly> assembly = LoadPlugInAssemblies();
            List<IPlugin> plugins = GetPlugIns(assembly);
            foreach (IPlugin p in plugins)
            {
                if (p.Suffix == pSuffix)
                    return p;
            }
            return null;
        }

        private static List<Assembly> LoadPlugInAssemblies()
        {
            DirectoryInfo dInfo = new DirectoryInfo(Path.Combine(Environment.CurrentDirectory, PluginDllLocation));
            FileInfo[] files = dInfo.GetFiles("*.dll");
            List<Assembly> plugInAssemblyList = new List<Assembly>();

            if (null != files)
            {
                foreach (FileInfo file in files)
                {
                    plugInAssemblyList.Add(Assembly.LoadFile(file.FullName));
                }
            }

            return plugInAssemblyList;
        }

        static List<IPlugin> GetPlugIns(List<Assembly> assemblies)
        {
            List<Type> availableTypes = new List<Type>();

            foreach (Assembly currentAssembly in assemblies)
                availableTypes.AddRange(currentAssembly.GetTypes());

            // get a list of objects that implement the IPlugin interface AND 
            // have the PluginAttribute
            List<Type> clientList = availableTypes.FindAll(delegate (Type t)
            {
                List<Type> interfaceTypes = new List<Type>(t.GetInterfaces());
                object[] arr = t.GetCustomAttributes(typeof(PluginAttribute), true);
                return !(arr == null || arr.Length == 0)
                    && arr.FirstOrDefault(x => ((PluginAttribute)x).Version == Version) != null
                    && interfaceTypes.Contains(typeof(IPlugin))
                    ;
            });

            // convert the list of Objects to an instantiated list of IPlugin
            return clientList.ConvertAll<IPlugin>(delegate (Type t) { return Activator.CreateInstance(t) as IPlugin; });
        }
    }
}
